@@ -36,17 +36,20 @@ import glib
 import gobject
 import gtk
 import pango
-import vte
+import logging
 import urllib
 import fcntl
 import posix
 import time
 import thread
+import vte
 import re
+import tempfile
+import gio
 
-from DebPackage import DebPackage, Cache
+from DebPackage import DebPackage
 from SimpleGtkbuilderApp import SimpleGtkbuilderApp
-from apt.progress import InstallProgress
+from apt.progress.base import InstallProgress
 from GDebiCommon import GDebiCommon, utf8
 from gettext import gettext as _
 
@@ -54,6 +57,15 @@ from gettext import gettext as _
 # is happening 
 GDEBI_TERMINAL_TIMEOUT=4*60.0
 
+# HACK - there are two ubuntu specific patches, one for VTE, one
+#        for gksu
+UBUNTU=False
+try:
+    import lsb_release
+    UBUNTU = lsb_release.get_distro_information()['ID'] == 'Ubuntu'
+except Exception, e:
+    pass
+
 class GDebi(SimpleGtkbuilderApp, GDebiCommon):
 
     def __init__(self, datadir, options, file=""):
@@ -61,7 +73,8 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         localesApp="gdebi"
         localesDir="/usr/share/locale"
 
-        SimpleGtkbuilderApp.__init__(self, path=datadir+"/gdebi.ui")
+        SimpleGtkbuilderApp.__init__(
+            self, path=datadir+"/gdebi.ui", domain="gdebi")
         # use a nicer default icon
         icons = gtk.icon_theme_get_default()
         try:
@@ -71,6 +84,15 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         except Exception, e:
           print "Error loading logo"
           pass
+
+        # Launchpad integration
+        try:
+            import LaunchpadIntegration
+            menu = self.builder.get_object('menuitem4_menu')
+            LaunchpadIntegration.set_sourcepackagename ("gdebi")
+            LaunchpadIntegration.add_items(menu, 1, True, False)
+        except ImportError:
+            pass
   
         # setup status
         self.context=self.statusbar_main.get_context_id("context_main_window")
@@ -89,8 +111,11 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
 
         # show what we have
         self.window_main.show()
+
+        # Check file with gio
+        file = self.gio_copy_in_place(file)
+
         #self.vte_terminal.set_font_from_string("monospace 10")
-        
         self.cprogress = self.CacheProgressAdapter(self.progressbar_cache)
         if not self.openCache():
             self.show_alert(gtk.MESSAGE_ERROR, self.error_header, self.error_body)
@@ -113,6 +138,9 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         column.add_attribute(render, "text", 0)
         self.treeview_files.append_column(column)
 
+        # empty config
+        self.synaptic_config = apt_pkg.Configuration()
+
         if file != "" and os.path.exists(file):
             self.window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
             while gtk.events_pending(): gtk.main_iteration()        
@@ -121,6 +149,48 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         
         self.window_main.set_sensitive(True)
 
+    def gio_progress_callback(self, bytes_read, bytes_total):
+        self.progressbar_download.set_fraction(bytes_read/float(bytes_total))
+        while gtk.events_pending():
+            gtk.main_iteration()
+
+    def on_button_cancel_download_clicked(self, button, cancellable):
+        cancellable.cancel()
+
+    def gio_copy_in_place(self, file):
+        "helper that copies the file to the local system via gio"
+        gio_file = gio.File(file)
+        if (gio_file.get_uri_scheme() == "file"):
+            return file
+        if (os.getuid()==0):
+            self.show_alert(gtk.MESSAGE_ERROR, 
+                            _("Can not download as root"),
+                            _("Remote packages can not be downloaded when "
+                              "running as root. Please try again as a "
+                              "normal user."))
+            sys.exit(1)
+        # Download the file
+        temp_file_name = os.path.join(tempfile.mkdtemp(),os.path.basename(file))
+        gio_dest = gio.File(temp_file_name)
+        try:
+            # download
+            gio_cancellable = gio.Cancellable()
+            self.button_cancel_download.connect("clicked", self.on_button_cancel_download_clicked, gio_cancellable)
+            self.dialog_gio_download.set_transient_for(self.window_main)
+            self.dialog_gio_download.show()
+            self.label_action.set_text(_("Downloading package"))
+            if gio_file.copy(gio_dest, self.gio_progress_callback, 0,
+                             gio_cancellable):
+                file = gio_dest.get_path()
+            self.dialog_gio_download.hide()
+        except Exception, e:
+            self.show_alert(gtk.MESSAGE_ERROR, 
+                            _("Download failed"),
+                            _("Downloading the package failed: "
+                              "file '%s' '%s'") % (file, e))
+            sys.exit(1)
+        return file
+
     def _get_file_path_from_dnd_dropped_uri(self, uri):
         """ helper to get a useful path from a drop uri"""
         path = urllib.url2pathname(uri) # escape special chars
@@ -135,7 +205,11 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         return path
     
     def on_menuitem_quit_activate(self, widget):
-        gtk.main_quit()
+        try:
+            gtk.main_quit()
+        except:
+            # if we are outside of the main loop, just exit
+            sys.exit(0)
 
     def on_window_main_drag_data_received(self, widget, context, x, y,
                                           selection, target_type, timestamp):
@@ -161,10 +235,10 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
 
         # set window title
         self.window_main.set_title(_("Package Installer - %s") % 
-                                   self._deb.pkgName)
+                                   self._deb.pkgname)
 
         # set name and ungrey some widgets
-        self.label_name.set_markup(self._deb.pkgName)
+        self.label_name.set_markup(self._deb.pkgname)
         self.notebook_details.set_sensitive(True)
         self.hbox_main.set_sensitive(True)
 
@@ -212,7 +286,7 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         self.label_maintainer.set_text(utf8(self._deb["Maintainer"]))
         self.label_priority.set_text(self._deb["Priority"])
         self.label_section.set_text(utf8(self._deb["Section"]))
-        self.label_size.set_text(self._deb["Installed-Size"] + " KB")
+        self.label_size.set_text(self._deb["Installed-Size"] + " KiB")
 
         # set file list
         store = gtk.TreeStore(str)
@@ -234,11 +308,12 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         self.textview_file_content.modify_font(font_desc)
 
         # check the deps
-        if not self._deb.checkDeb():
-            self.label_status.set_markup("<span foreground=\"red\" weight=\"bold\">"+
-                                         _("Error: ") +
-                                         glib.markup_escape_text(self._deb._failureString) +
-                                         "</span>")
+        if not self._deb.check():
+            self.label_status.set_markup(
+                "<span foreground=\"red\" weight=\"bold\">"+
+                _("Error: ") +
+                glib.markup_escape_text(self._deb._failure_string) +
+                "</span>")
 	    self.button_install.set_label(_("_Install Package"))
 
             self.button_install.set_sensitive(False)
@@ -248,9 +323,9 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
 
         # set version_info_{msg,title} strings
         self.compareDebWithCache()
-        self.getChanges()
+        self.get_changes()
 
-        if self._deb.compareToVersionInCache() == DebPackage.VERSION_SAME:
+        if self._deb.compare_to_version_in_cache() == DebPackage.VERSION_SAME:
             self.label_status.set_text(_("Same version is already installed"))
             self.button_install.set_label(_("_Reinstall Package"))
             self.button_install.grab_default()
@@ -357,7 +432,7 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
         self.window_main.set_sensitive(False)
         self.openCache()
         if self._deb:
-            self.open(self._deb.file)
+            self.open(self._deb.filename)
         self.window_main.set_sensitive(True)
 
     def on_about_activate(self, widget):
@@ -388,7 +463,7 @@ class GDebi(SimpleGtkbuilderApp, GDebiCommon):
             secondary = _("Malicious software can damage your data "
                           "and take control of your system.\n\n"
                           "The packages below are not authenticated and "
-                          "could therefor be of malicious nature.")
+                          "could therefore be of malicious nature.")
             msg = "<big><b>%s</b></big>\n\n%s" % (primary, secondary)
             dialog = gtk.MessageDialog(parent=self.dialog_deb_install,
                                        flags=gtk.DIALOG_MODAL,
@@ -418,12 +493,21 @@ It is a possible security risk to install packages files manually.
 Install software from trustworthy software distributors only.
 """)
         if os.getuid() != 0:
-            os.execl("/usr/bin/gksu", "gksu", "--desktop",
-                     "/usr/share/applications/gdebi.desktop",
-                     "--message","<big><b>%s</b></big>\n\n%s" % (msg_hdr,msg_bdy),
-                     "--always-ask-pass",
-                     "--", "gdebi-gtk", "--non-interactive",
-                     self._deb.file)
+
+            # build command and argument lists
+            gksu_cmd = "/usr/bin/gksu"
+            gksu_args = ["gksu", "--desktop",
+                         "/usr/share/applications/gdebi.desktop",
+                         "--message",
+                         "<big><b>%s</b></big>\n\n%s" % (msg_hdr,msg_bdy)]
+            gdebi_args = ["--", "gdebi-gtk", "--non-interactive",
+                          self._deb.filename]
+            # check if we run on ubuntu and always ask for the password
+            # there - we would like to do that on debian too, but this
+            # gksu patch is only available on ubuntu currently unfortunately
+            if UBUNTU:
+                    gksu_args.append("--always-ask-pass")
+            os.execv(gksu_cmd, gksu_args+gdebi_args)
 
         if not self.try_acquire_lock():
             self.statusbar_main.push(self.context,
@@ -436,6 +520,17 @@ Install software from trustworthy software distributors only.
         self.button_deb_install_close.set_sensitive(False)
         # clear terminal
         #self.vte_terminal.feed(str(0x1b)+"[2J")
+        
+        # Get whether we auto close from synaptic's config file and
+        # update the toggle button as neccessary
+        config = apt_pkg.Configuration()
+        if os.path.isfile("/root/.synaptic/synaptic.conf"):
+            apt_pkg.read_config_file(config, "/root/.synaptic/synaptic.conf")
+        else:
+            config["Synaptic::closeZvt"] = "false"
+        self.synaptic_config = config.subtree("Synaptic")
+        self.checkbutton_autoclose.set_active(self.synaptic_config.find_b("closeZvt"))
+        
         self.dialog_deb_install.set_transient_for(self.window_main)
         self.dialog_deb_install.show_all()
 
@@ -479,10 +574,11 @@ Install software from trustworthy software distributors only.
                 return 
     
         # install the package itself
+        self.dialog_deb_install.set_title(self.window_main.get_title())
         self.label_action.set_markup("<b><big>" +
-                                     _("Installing %s") % self._deb.pkgName+
+                                     _("Installing %s") % self._deb.pkgname+
                                      "</big></b>")
-        dprogress = self.DpkgInstallProgress(self._deb.file,
+        dprogress = self.DpkgInstallProgress(self._deb.filename,
                                              self.label_install_status,
                                              self.progressbar_install,
                                              self.vte_terminal,
@@ -498,9 +594,9 @@ Install software from trustworthy software distributors only.
             self.on_button_deb_install_close_clicked(None)
         self.label_action.set_markup("<b><big>"+_("Installation finished")+"</big></b>")
         if dprogress.exitstatus == 0:
-            self.label_install_status.set_markup("<i>"+_("Package '%s' was installed") % os.path.basename(self._deb.file)+"</i>")
+            self.label_install_status.set_markup("<i>"+_("Package '%s' was installed") % os.path.basename(self._deb.filename)+"</i>")
         else:
-            self.label_install_status.set_markup("<b>"+_("Failed to install package '%s'") % os.path.basename(self._deb.file)+"</b>")
+            self.label_install_status.set_markup("<b>"+_("Failed to install package '%s'") % os.path.basename(self._deb.filename)+"</b>")
             self.expander_install.set_expanded(True)
         self.statusbar_main.push(self.context,_("Installation complete"))
         # FIXME: Doesn't stop notifying
@@ -508,16 +604,23 @@ Install software from trustworthy software distributors only.
 
         # reopen the cache, reread the file
         self.openCache()
-        if self._cache._depcache.BrokenCount > 0:
+        if self._cache._depcache.broken_count > 0:
             err_header = _("Failed to completely install all dependencies")
             err_body = _("To fix this run 'sudo apt-get install -f' in a "
                          "terminal window.")
             self.show_alert(gtk.MESSAGE_ERROR, err_header, err_body)
-        self.open(self._deb.file)
+        self.open(self._deb.filename)
         
     def on_button_deb_install_close_clicked(self, widget):
+        # Set the autoclose option when we close
+        autoclose = self.checkbutton_autoclose.get_active()
+        self.synaptic_config["closeZvt"] = str(autoclose).lower()
+        self.write_synaptic_config_file(self.synaptic_config,
+                                        "/root/.synaptic/synaptic.conf")
         # FIXME: doesn't turn it off
         #self.window_main.set_property("urgency-hint", 0)
+        if hasattr(self, "_gio_cancellable"):
+            self._gio_cancellable.cancel()
         self.dialog_deb_install.hide()
         self.window_main.set_sensitive(True)
     
@@ -563,6 +666,13 @@ Install software from trustworthy software distributors only.
             return True
         return False
         
+    def write_synaptic_config_file(self, config, path):
+        if not os.path.exists(path):
+            return
+        config_file = open(path, "w")
+        config_file.write(config.dump())
+        config_file.close()
+        
     # embedded classes
     class DpkgInstallProgress(object):
         def __init__(self, debfile, status, progress, term, expander):
@@ -574,13 +684,17 @@ Install software from trustworthy software distributors only.
             self.time_last_update = time.time()
             self.term_expander.set_expanded(False)
         def commit(self):
-            def finish_dpkg(term, pid, status, lock):
-                " helper "
+            def finish_dpkg(term, lock):
+                """ helper that is run when dpkg finishes """
+                status = term.get_child_exit_status()
                 self.exitstatus = posix.WEXITSTATUS(status)
                 #print "dpkg finished %s %s" % (pid,status)
                 #print "exit status: %s" % self.exitstatus
                 #print "was signaled %s" % posix.WIFSIGNALED(status)
-                lock.release()
+                try:
+                    lock.release()
+                except:
+                    logging.exception("lock.release failed")
 
             # get a lock
             lock = thread.allocate_lock()
@@ -599,7 +713,12 @@ Install software from trustworthy software distributors only.
 
             # the command
             cmd = "/usr/bin/dpkg"
-            argv = [cmd,"--status-fd", "%s"%writefd, "-i", self.debfile]
+            argv = [cmd, "--auto-deconfigure"]
+            # ubuntu supports VTE_PTY_KEEP_FD, see 
+            # https://bugzilla.gnome.org/320128 for the upstream bug
+            if UBUNTU:
+                argv += ["--status-fd", "%s"%writefd]
+            argv += ["-i", self.debfile]
             env = ["VTE_PTY_KEEP_FD=%s"% writefd,
                    "DEBIAN_FRONTEND=gnome",
                    "APT_LISTCHANGES_FRONTEND=gtk"]
@@ -609,8 +728,7 @@ Install software from trustworthy software distributors only.
             #print self.term
 
             # prepare for the fork
-            reaper = vte.reaper_get()
-            signal_id = reaper.connect("child-exited", finish_dpkg, lock)
+            self.term.connect("child-exited", finish_dpkg, lock)
             pid = self.term.fork_command(command=cmd, argv=argv, envv=env)
             read = ""
             while lock.locked():
@@ -643,7 +761,6 @@ Install software from trustworthy software distributors only.
                     (self.time_last_update + GDEBI_TERMINAL_TIMEOUT) < time.time()):
                   self.term_expander.set_expanded(True)
             self.progress.set_fraction(1.0)
-            reaper.disconnect(signal_id)
     
     class InstallProgressAdapter(InstallProgress):
         def __init__(self,progress,term,label,term_expander):
@@ -654,13 +771,15 @@ Install software from trustworthy software distributors only.
             self.finished = False
             self.action = label
             self.time_last_update = time.time()
-            reaper = vte.reaper_get()
-            reaper.connect("child-exited",self.child_exited)
+            self.term.connect("child-exited", self.child_exited)
             self.env = ["VTE_PTY_KEEP_FD=%s"% self.writefd,
                         "DEBIAN_FRONTEND=gnome",
                         "APT_LISTCHANGES_FRONTEND=gtk"]
-        def child_exited(self,term, pid, status):
-            #print "child_exited: %s %s %s %s" % (self,term,pid,status)
+        def child_exited(self,term):
+            status = term.get_child_exit_status()
+            #print "apt finished %s" % status
+            #print "exit status: %s" % posix.WEXITSTATUS(status)
+            #print "was signaled %s" % posix.WIFSIGNALED(status)
             self.apt_status = status
             self.finished = True
         def error(self, pkg, errormsg):
@@ -669,18 +788,19 @@ Install software from trustworthy software distributors only.
         def conffile(self, current, new):
             # FIXME: display a msg or expand term
             self.term_expander.set_expanded(True)
-        def startUpdate(self):
+        def start_update(self):
             #print "startUpdate"
-            apt_pkg.PkgSystemUnLock()
+            apt_pkg.pkgsystem_unlock()
             self.action.set_markup("<i>"+_("Installing dependencies...")+"</i>")
             self.progress.set_fraction(0.0)
             self.progress.set_text("")
-        def statusChange(self, pkg, percent, status):
+        def status_change(self, pkg, percent, status):
+            #print "status change", pkg, percent, status
             self.progress.set_fraction(percent/100.0)
             self.progress.set_text(status)
             self.time_last_update = time.time()
-        def updateInterface(self):
-            InstallProgress.updateInterface(self)
+        def update_interface(self):
+            InstallProgress.update_interface(self)
             while gtk.events_pending():
                 gtk.main_iteration()
             if (not self.term_expander.get_expanded() and 
@@ -696,35 +816,36 @@ Install software from trustworthy software distributors only.
                     (key, value) = env.split("=")
                     os.environ[key] = value
             return pid
-        def waitChild(self):
+        def wait_child(self):
             while not self.finished:
-                self.updateInterface()
+                self.update_interface()
             return self.apt_status
         
-    class FetchProgressAdapter(apt.progress.FetchProgress):
+    class FetchProgressAdapter(apt.progress.base.AcquireProgress):
         def __init__(self,progress,action,main):
-            #print "FetchProgressAdapter.__init__()"
+            super(GDebi.FetchProgressAdapter, self).__init__()
             self.progress = progress
             self.action = action
             self.main = main
         def start(self):
-            #print "start()"
+            super(GDebi.FetchProgressAdapter, self).start()
             self.action.set_markup("<i>"+_("Downloading additional package files...")+"</i>")
             self.progress.set_fraction(0)
         def stop(self):
             #print "stop()"
             pass
-        def pulse(self):
-            at_item = min(self.currentItems + 1, self.totalItems)
-            if self.currentCPS > 0:
-                self.progress.set_text(_("File %s of %s at %sB/s") % (at_item,self.totalItems,apt_pkg.SizeToStr(self.currentCPS)))
+        def pulse(self, owner):
+            super(GDebi.FetchProgressAdapter, self).pulse(owner)
+            at_item = min(self.current_items + 1, self.total_items)
+            if self.current_cps > 0:
+                self.progress.set_text(_("File %s of %s at %sB/s") % (at_item,self.total_items,apt_pkg.size_to_str(self.current_cps)))
             else:
-                self.progress.set_text(_("File %s of %s") % (at_item,self.totalItems))
-            self.progress.set_fraction(self.currentBytes/self.totalBytes)
+                self.progress.set_text(_("File %s of %s") % (at_item,self.total_items))
+            self.progress.set_fraction(self.current_bytes/self.total_bytes)
             while gtk.events_pending():
                 gtk.main_iteration()
             return True
-        def mediaChange(self, medium, drive):
+        def media_change(self, medium, drive):
             #print "mediaChange %s %s" % (medium, drive)
             msg = _("Please insert '%s' into the drive '%s'" % (medium,drive))
             dialog = gtk.MessageDialog(parent=self.main,
@@ -739,12 +860,13 @@ Install software from trustworthy software distributors only.
                 return True
             return False
 
-    class CacheProgressAdapter(apt.progress.FetchProgress):
+    class CacheProgressAdapter(apt.progress.base.OpProgress):
         def __init__(self, progressbar):
             self.progressbar = progressbar
-        def update(self, percent):
+        def update(self, percent=None):
             self.progressbar.show()
-            self.progressbar.set_fraction(percent/100.0)
+            if percent:
+                self.progressbar.set_fraction(percent/100.0)
             #self.progressbar.set_text(self.op)
             while gtk.events_pending():
                 gtk.main_iteration()
@@ -757,13 +879,13 @@ if __name__ == "__main__":
     pkgs = ["cw"]
     for pkg in pkgs:
         print "installing %s" % pkg
-        app._cache[pkg].markInstall()
+        app._cache[pkg].mark_install()
 
     for pkg in app._cache:
-        if pkg.markedInstall or pkg.markedUpgrade:
+        if pkg.marked_install or pkg.marked_upgrade:
             print pkg.name
 
-    apt_pkg.PkgSystemLock()
+    apt_pkg.pkgsystem_lock()
     app.dialog_deb_install.set_transient_for(app.window_main)
     app.dialog_deb_install.show_all()
  
