@@ -31,9 +31,10 @@ import time
 import thread
 
 from gettext import gettext as _
+from re import findall
 
-from DebPackage import DebPackage, Cache
-from DscSrcPackage import DscSrcPackage
+from apt.cache import Cache
+from DebPackage import DebPackage, DscSrcPackage
 
 from subprocess import PIPE, Popen, call
 
@@ -43,15 +44,15 @@ class GDebiCli(object):
         # fixme, do graphic cache check
         self.options = options
         if options.quiet:
-            tp = apt.progress.OpProgress()
+            tp = apt.progress.base.OpProgress()
         else:
-            tp = apt.progress.OpTextProgress()
+            tp = apt.progress.text.OpProgress()
         # set architecture to architecture in root-dir
         if options.rootdir and os.path.exists(options.rootdir+"/usr/bin/dpkg"):
             arch = Popen([options.rootdir+"/usr/bin/dpkg",
                           "--print-architecture"], stdout=PIPE).communicate()[0]
             if arch:
-                apt_pkg.Config.Set("APT::Architecture",arch.strip())
+                apt_pkg.config.set("APT::Architecture",arch.strip())
         if options.apt_opts:
             for o in options.apt_opts:
                 if o.find('=') < 0:
@@ -59,7 +60,7 @@ class GDebiCli(object):
                     sys.exit(1)
                 (name, value) = o.split('=', 1)
                 try:
-                    apt_pkg.Config.Set(name, value)
+                    apt_pkg.config.set(name, value)
                 except:
                     sys.stderr.write(_("Couldn't set APT option %s to %s\n") % (name, value))
                     sys.exit(1)
@@ -67,11 +68,12 @@ class GDebiCli(object):
 
     def open(self, file):
         try:
-            if file.endswith(".deb"):
-                self._deb = DebPackage(self._cache, file)
+            if (file.endswith(".deb") or 
+                "Debian binary package" in Popen(["file", file], stdout=PIPE).communicate()[0]):
+                self._deb = DebPackage(file, self._cache)
             elif (file.endswith(".dsc") or
                   os.path.basename(file) == "control"):
-                self._deb = DscSrcPackage(self._cache, file)
+                self._deb = DscSrcPackage(file, self._cache)
             else:
                 sys.stderr.write(_("Unknown package type '%s', exiting\n") % file)
                 sys.exit(1)
@@ -82,9 +84,9 @@ class GDebiCli(object):
                            "of the file.\n"))
             sys.exit(1)
         # check the deps
-        if not self._deb.checkDeb():
+        if not self._deb.check():
             sys.stderr.write(_("This package is uninstallable\n"))
-            sys.stderr.write(self._deb._failureString + "\n")
+            sys.stderr.write(self._deb._failure_string + "\n")
             return False
         return True
             
@@ -96,7 +98,7 @@ class GDebiCli(object):
 
     def show_dependencies(self):
         # show what changes
-        (install, remove, unauthenticated) = self._deb.requiredChanges
+        (install, remove, unauthenticated) = self._deb.required_changes
         if len(unauthenticated) > 0:
             print _("The following packages are UNAUTHENTICATED: ")
             for pkgname in unauthenticated:
@@ -114,29 +116,32 @@ class GDebiCli(object):
 
     def install(self):
         # install the dependecnies
-        (install,remove,unauthenticated) = self._deb.requiredChanges
+        (install,remove,unauthenticated) = self._deb.required_changes
         if len(install) > 0 or len(remove) > 0:
-            fprogress = apt.progress.TextFetchProgress()
-            iprogress = apt.progress.InstallProgress()
+            fprogress = apt.progress.text.AcquireProgress()
+            iprogress = apt.progress.base.InstallProgress()
             try:
                 res = self._cache.commit(fprogress,iprogress)
+            except apt.cache.FetchFailedException, e:
+                sys.stderr.write(_("Error during install: '%s'") % e)
+                return False
             except SystemError, e:
                 sys.stderr.write(_("Error during install: '%s'") % e)
                 return False
 
         # install the package itself
-        if self._deb.file.endswith(".dsc"): 
+        if self._deb.filename.endswith(".dsc"): 
             # FIXME: add option to only install build-dependencies
             #        (or build+install the deb) and then enable
             #        this code
-            #dir = self._deb.pkgName + "-" + apt_pkg.UpstreamVersion(self._deb["Version"])
-            #os.system("dpkg-source -x %s" % self._deb.file)
+            #dir = self._deb.pkgname + "-" + apt_pkg.UpstreamVersion(self._deb["Version"])
+            #os.system("dpkg-source -x %s" % self._deb.filename)
             #os.system("cd %s && dpkg-buildpackage -b -uc" % dir)
             #for i in self._deb.binaries:
             #    os.system("gdebi %s_%s_*.deb" % (i,self._deb["Version"]))
             pass
         else:
-            ret = call(["dpkg","-i",self._deb.file])
+            ret = call(["dpkg","--auto-deconfigure", "-i",self._deb.filename])
             if (ret != 0):
                 return False
         return True
@@ -146,8 +151,13 @@ if __name__ == "__main__":
     app = GDebiCli()
     if not app.open(sys.argv[1]):
         sys.exit(1)
-    print _("Do you want to install the software package? [y/N]:"),
+    msg =  _("Do you want to install the software package? [y/N]:")
+    print msg,
     sys.stdout.flush()
     res = sys.stdin.readline()
-    if res.startswith("y") or res.startswith("Y"):
+    try:
+        c = findall("\[(\S+)/\S+\]", msg)[0].lower()
+    except IndexError:
+        c = "y"
+    if res.lower().startswith(c):
         app.install()
