@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright (c) 2005-2007 Martin Böhm
-# Copyright (c) 2008 Canonical Ltd
+# Copyright (c) 2008-2009 Canonical Ltd
 #
 # AUTHOR:
 # Martin Böhm <martin.bohm@ubuntu.com>
@@ -37,19 +37,15 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from PyQt4 import uic
 
-from DebPackage import DebPackage, Cache
+from apt.cache import Cache
+from DebPackage import DebPackage
+
 import gettext
-from GDebiCommon import GDebiCommon
+from GDebiCommon import GDebiCommon, utf8, _
 from KDEAptDialogs import *
 
-def _(str):
-    return unicode(gettext.gettext(str), 'UTF-8')
 def __(catalog,str):
     return unicode(gettext.dgettext(catalog, str), 'UTF-8')
-def utf8(str):
-  if isinstance(str, unicode):
-      return str
-  return unicode(str, 'UTF-8')
 
 def loadUi(file, parent):
     """load local file if possible else use installed file"""
@@ -101,7 +97,7 @@ class DumbTerminal(QTextEdit):
             #print "sent backspace"
             os.write(self.installProgress.master_fd, chr(8))
             return
-        # do nothing for events like "shift" 
+        # do nothing for events like "shift"
         if not ev.text():
             return
         # now sent the key event to the termianl as utf-8
@@ -129,7 +125,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         GDebiKDEDialog.__init__(self,parent)
         GDebiCommon.__init__(self,datadir,options,file)
         # load the icon
-        self.setWindowIcon(KIcon("adept_installer"))
+        self.setWindowIcon(KIcon("application-x-deb"))
         # first, we load all the default descriptions -- pyuic doesn't use
         # gettext as default (FIXME, copy code from language-selector)
         self.textLabel1.setText(_("Package:"))
@@ -182,8 +178,8 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             return False
 
         # set name
-        self.setWindowTitle(_("Package Installer - %s") % self._deb.pkgName)
-        self.textLabel1_3.setText(self._deb.pkgName)
+        self.setWindowTitle(_("Package Installer - %s") % self._deb.pkgname)
+        self.textLabel1_3.setText(self._deb.pkgname)
 
         # set description
         buf = self.DecriptionEdit
@@ -221,16 +217,16 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             buf.setText(_("No description is available"))
 
         # check deps
-        if not self._deb.checkDeb():
+        if not self._deb.check():
             self.installButton.setEnabled(False)
-            self.textLabel1_3_2.setText("<font color=\"#ff0000\"> Error: " + self._deb._failureString + "</font>")
+            self.textLabel1_3_2.setText(utf8("<font color=\"#ff0000\"> Error: " + self._deb._failure_string + "</font>"))
             self.detailsButton.hide()
             return False
 
         # set version_info_{msg,title} strings
         self.compareDebWithCache()
 
-        if self._deb.compareToVersionInCache() == DebPackage.VERSION_SAME:
+        if self._deb.compare_to_version_in_cache() == DebPackage.VERSION_SAME:
             #self.textLabel1_3_2.setText(_("Same version is already installed"))
             self.installButton.setText(_("&Reinstall Package"))
             self.installButton.setIcon(KIcon("view-refresh"))
@@ -240,7 +236,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             #self.button_details.hide()
 
         # load changes into (self.install, self.remove, self.unauthenticated)
-        self.getChanges()
+        self.get_changes()
 
         if self.version_info_title != "" and self.version_info_msg != "":
             self.infoIcon.setPixmap(DesktopIcon("dialog-information"))
@@ -257,16 +253,16 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.DetailsMaintainer.setText(utf8(self._deb["Maintainer"]))
         self.DetailsPriority.setText(self._deb["Priority"])
         self.DetailsSection.setText(utf8(self._deb["Section"]))
-        self.DetailsSize.setText(self._deb["Installed-Size"] + " KB")
+        self.DetailsSize.setText(self._deb["Installed-Size"] + " KiB")
 
         # set filelist
         buf = self.IncFilesEdit
         buf.setText("\n".join(self._deb.filelist))
 
-        if not self._deb.checkDeb():
+        if not self._deb.check():
             self.installButton.setText(_("&Install Package"))
 
-        if self._deb.compareToVersionInCache() == DebPackage.VERSION_SAME:
+        if self._deb.compare_to_version_in_cache() == DebPackage.VERSION_SAME:
             self.installButton.setText(_("Re&install Package"))
 
     def cancelButtonClicked(self):
@@ -274,7 +270,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
 
     def detailsButtonClicked(self):
         changedList = QStringList()
-        (install, remove, unauthenticated) = self._deb.requiredChanges
+        (install, remove, unauthenticated) = self._deb.required_changes
         for i in install:
             changedList.append(_("To be installed: %s") % i)
         for r in remove:
@@ -292,7 +288,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             else:
                 executable = "/usr/bin/gdebi-kde"
             print "executable " + executable
-            os.execl("/usr/bin/kdesudo", "kdesudo", executable + " -n ", self._deb.file)
+            os.execl("/usr/bin/kdesudo", "kdesudo", executable + " -n ", self._deb.filename)
             self.kapp.exit()
 
         if not self.try_acquire_lock():
@@ -344,7 +340,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
 
         # install the package itself
         #self.label_action.set_markup("<b><big>"+_("Installing package file")+"</big></b>")
-        dprogress = KDEDpkgInstallProgress(self._deb.file,
+        dprogress = KDEDpkgInstallProgress(self._deb.filename,
                                              self.installDialog.installingLabel,
                                              self.installDialog.installationProgress,
                                              self.installDialog.konsole, self.installDialog)
@@ -355,9 +351,9 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         #self.button_deb_install_close.grab_default()
         self.installDialog.setWindowTitle(_("Installation finished"))
         if dprogress.exitstatus == 0:
-            self.installDialog.installingLabel.setText(_("<b>" + "Package '%s' was installed" + "</b>") % self._deb.pkgName)
+            self.installDialog.installingLabel.setText(_("<b>" + "Package '%s' was installed" + "</b>") % self._deb.pkgname)
         else:
-            self.installDialog.installingLabel.setText("<b>"+_("Failed to install package '%s'") % os.path.basename(self._deb.file)+"</b>")
+            self.installDialog.installingLabel.setText("<b>"+_("Failed to install package '%s'") % os.path.basename(self._deb.filename)+"</b>")
             self.installDialog.konsoleFrame.show()
         #self.statusbar_main.push(self.context,_("Installation complete"))
         # FIXME: Doesn't stop notifying
@@ -366,7 +362,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         # reopen the cache, reread the file, FIXME: add progress reporting
         #self._cache = Cache(self.cprogress)
         self._cache = Cache()
-        if self._cache._depcache.BrokenCount > 0:
+        if self._cache._depcache.broken_count > 0:
             header = _("Failed to completely install all dependencies")
             text = _("To fix this run 'sudo apt-get install -f' in a "
                          "terminal window.")
